package de.intevation.flys.exports;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import org.jfree.data.xy.DefaultXYDataset;
import org.jfree.data.xy.XYDataset;

import de.intevation.artifacts.Artifact;

import de.intevation.flys.model.Gauge;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.DischargeTables;
import de.intevation.flys.artifacts.model.GaugesFactory;


/**
 * An OutGenerator that generates discharge curves.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class DischargeCurveGenerator extends XYChartGenerator {

    /** The logger used in this generator.*/
    private static Logger logger =
        Logger.getLogger(DischargeCurveGenerator.class);


    /** The Dataset that contains the data for the chart.*/
    protected DefaultXYDataset dataset;


    public DischargeCurveGenerator() {
        super();

        this.dataset = new DefaultXYDataset();
    }


    protected String getChartTitle() {
        return "Abflusskurven";
    }


    protected String getXAxisLabel() {
        return "Q[m³/s]";
    }


    protected String getYAxisLabel() {
        return "W[cm]";
    }


    protected XYDataset getXYDataset() {
        return dataset;
    }


    public void doOut(Artifact artifact, String facet, Document attr) {
        logger.debug("DischargeCurveGenerator.doOut: " + facet);

        if (!(artifact instanceof FLYSArtifact)) {
            logger.error("Artifact is no instance of FLYSArtifact.");
            return;
        }

        FLYSArtifact flysArtifact = (FLYSArtifact) artifact;

        String river = (String) flysArtifact.getData("river").getValue();
        String from  = (String) flysArtifact.getData("ld_from").getValue();
        String to    = (String) flysArtifact.getData("ld_to").getValue();

        double f = Double.parseDouble(from);
        double t = Double.parseDouble(to);

        List<double[]> ranges = new ArrayList<double[]>();
        ranges.add(new double [] { f, t });

        List<Gauge> gauges   = GaugesFactory.getGauges(river);
        List<Gauge> filtered = GaugesFactory.filterRanges(gauges, ranges);

        if (logger.isDebugEnabled()) {
            int numGauges   = gauges != null ? gauges.size() : 0;
            int numFiltered = filtered != null ? filtered.size() : 0;

            logger.debug("++++++++++++++++++++");
            logger.debug("Search gauges for river: " + river);
            logger.debug("-> ... from range: " + from);
            logger.debug("-> ... to range: " + to);
            logger.debug("-> Found " + numGauges + " gauges in total.");
            logger.debug("-> Found " + numFiltered + " gauges after filtering.");
            logger.debug("++++++++++++++++++++");
        }

        String[] gaugeNames = new String[filtered.size()];
        int idx = 0;
        for (Gauge gauge: filtered) {
            gaugeNames[idx++] = gauge.getName();
        }

        DischargeTables dt = new DischargeTables(river, gaugeNames);
        Map<String, double[][]> gaugeValues = dt.getValues(100d);

        for (String gauge: gaugeNames) {
            double[][] values = gaugeValues.get(gauge);
            dataset.addSeries(gauge, values);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
